namespace blepo
{

// Parameters for LKA() and 
// LKARegions() functions
struct LKAParameters
{
  int min_num_pixels; // minimum number of pixels in region for tracking
  int num_iterations; // maximum number of iterations 

  float oob_threshold; // ratio of warped area going out of the image
  float max_disp_threshold; // maximum displacement allowed
  float min_disp_threshold; // minimum displacement for stopping iterations
  float max_aff_threshold; // maximum affine warp allowed
  float min_aff_threshold; // minimum affine warp for stopping iterations

  float det_threshold; // minimum determinant of T matrix of Tz = d
  float residue_threshold; // maximum residue allowed

  float shrink_width_ratio; // horizontal border width (ratio) to be removed
  float shrink_height_ratio; // vertical border width (ratio) to be removed
  float smooth_sigma; // gaussian sigma for smoothing
  float gradient_sigma; // gaussian sigma for gradient computation
  int gradient_kernel_length; // kernel length of gaussian
  bool skip_label_zero; // set to true for skipping label zero in mask image
                        // must be true for LKA() functions
                        // can be true or false for LKARegions() function
  bool origin_centroid; // set to true if parameters for each region are to be computed
                        // with origin at that region's centroid
                        // set to false for common origin at image's (0,0)

  int shrink_width;
  int shrink_height;

  // defalut values
  LKAParameters()
  {
    min_num_pixels = 25;
    num_iterations = 20;

    oob_threshold = 0.2f; 
    max_disp_threshold = 5.0f;
    min_disp_threshold = 0.05f;
    max_aff_threshold = 0.5f;
    min_aff_threshold = 0.005f;

    det_threshold = 1000.0f;
    residue_threshold = 3.0f;

    shrink_width_ratio = 0.05f;
    shrink_height_ratio = 0.05f;
    smooth_sigma = 0.8f;
    gradient_sigma = 0.8f;
    gradient_kernel_length = 7;
    skip_label_zero = true;
    origin_centroid = true;

    shrink_width = 10;
    shrink_height = 10; 
  }
};

// Result structure for LKA() and 
// LKARegions() functions
struct LKAResult
{
  
  int status;

  // status 0 mean region was tracked
  // status 1 means region was not found
  // status 2 means region determinant less than threshold
  // status 3 means region calculations reached max iterations
  // status 4 means region out of bounds
  // status 5 means region residue more than threshold
  // status 6 means region area is smaller than threshold 
  // status 7 means region is either near border or nonexistent

  MatFlt A;
  MatFlt d;
  float centroid_x;
  float centroid_y;
  float residue;
};


// Function to compute affine tracking of a rectangular patch 
// from img1 to img2 using the Lucas Kanade method. The rectangular 
// region patch, in img1, to be tracked is defined by the rect argument. 
// Forwards Additive implementation of the algorithm is used.
// z has all the 6 calculated affine paramaters and the 
// validity of tracking is returned in status. By default, affine 
// parameters are with respect to the centroid of the rectangular 
// region. Coordinates of the centroid are also returned.
// @author Braga Natarajan.

void LKA(
  const ImgFloat& img1,
  const ImgFloat& img2,
  const Rect& rect,
  const LKAParameters& params,
  const MatFlt& A_init,
  const MatFlt& d_init,
  LKAResult* result);


// Function to compute affine tracking of an arbitrarily shaped patch 
// from img1 to img2 using the Lucas Kanade method. The region patch,
// in img1, to be tracked is defined by the binary image mask. 
// Forwards Additive implementation of the algorithm is used.
// z has all the 6 calculated affine paramaters and the 
// validity of tracking is returned in status. By default, affine 
// parameters are with respect to the centroid of the arbitrarily 
// shaped region. Coordinates of the centroid are also returned.
// @author Braga Natarajan.

void LKA(
  const ImgFloat& img1,
  const ImgFloat& img2,
  const ImgBinary& mask,
  const LKAParameters& params,
  const MatFlt& A_init,
  const MatFlt& d_init,
  LKAResult* result);


// Function to compute affine tracking of multiple regions from img1
// to img2 using the Lucas Kanade method. The different regions, in
// img1, to be tracked are defined by the integer label image img_label. 
// Forwards Additive implementation of the algorithm is used.
// z_regions has the 6 calculated affine paramaters for all the
// regions indexed by label number and validity of tracking is returned 
// in status_regions. By default, the affine parameters for each region 
// are with respect to the centroid of that particular region. The 
// coordinates of the centroids for all regions are also returned.
// @author Braga Natarajan.

void LKARegions(
  const ImgFloat& img1,
  const ImgFloat& img2,
  const ImgInt& img_label,
  const LKAParameters& params,
  const std::vector<MatFlt>& A_init_regions,
  const std::vector<MatFlt>& d_init_regions,
  std::vector<LKAResult>* result_regions);

// functions under development

void LKAPre(
  const ImgFloat& img1,
  const ImgFloat& img2,
  const LKAParameters& params,
  ImgFloat* img1_smoothed,
  ImgFloat* img2_smoothed,
  ImgFloat* gradx,
  ImgFloat* grady);

void LKAPost(
  const ImgFloat& img1_smoothed,
  const ImgFloat& img2_smoothed,
  const ImgFloat& gradx,
  const ImgFloat& grady,
  const ImgBinary& mask,
  const LKAParameters& params,
  const MatFlt& A_init,
  const MatFlt& d_init,
  LKAResult* result);

};

